@@ -3,32 +3,34 @@ module Rails
     mattr_accessor :properties
     class << (@@properties = [])
       def names
-        map {|(name, )| name}
+        map &:first
       end
-      
+
       def value_for(property_name)
-        find {|(name, )| name == property_name}.last rescue nil
+        if property = assoc(property_name)
+          property.last
+        end
       end
     end
-  
+
     class << self #:nodoc:
       def property(name, value = nil)
         value ||= yield
-        properties << [name, value] if value 
+        properties << [name, value] if value
       rescue Exception
       end
 
-      def components
+      def frameworks
         %w( active_record action_pack active_resource action_mailer active_support )
       end
-      
-      def component_version(component)
-        require "#{component}/version"
-        "#{component.classify}::VERSION::STRING".constantize
+
+      def framework_version(framework)
+        require "#{framework}/version"
+        "#{framework.classify}::VERSION::STRING".constantize
       end
-    
-      def edge_rails_revision(info = svn_info)
-        info[/^Revision: (\d+)/, 1] || freeze_edge_version
+
+      def edge_rails_revision(info = git_info)
+        info[/commit ([a-z0-9-]+)/, 1] || freeze_edge_version
       end
 
       def freeze_edge_version
@@ -49,7 +51,7 @@ module Rails
       end
 
       alias inspect to_s
-      
+
       def to_html
         returning table = '<table>' do
           properties.each do |(name, value)|
@@ -65,10 +67,10 @@ module Rails
           @rails_vendor_root ||= "#{RAILS_ROOT}/vendor/rails"
         end
 
-        def svn_info
+        def git_info
           env_lang, ENV['LC_ALL'] = ENV['LC_ALL'], 'C'
           Dir.chdir(rails_vendor_root) do
-            silence_stderr { `svn info` }
+            silence_stderr { `git log -n 1` }
           end
         ensure
           ENV['LC_ALL'] = env_lang
@@ -82,25 +84,29 @@ module Rails
     property 'RubyGems version' do
       Gem::RubyGemsVersion
     end
-  
+
+    property 'Rack version' do
+      ::Rack.release
+    end
+
     # The Rails version.
     property 'Rails version' do
       Rails::VERSION::STRING
     end
-  
-    # Versions of each Rails component (Active Record, Action Pack, 
+
+    # Versions of each Rails framework (Active Record, Action Pack,
     # Active Resource, Action Mailer, and Active Support).
-    components.each do |component|
-      property "#{component.titlecase} version" do 
-        component_version(component)
+    frameworks.each do |framework|
+      property "#{framework.titlecase} version" do
+        framework_version(framework)
       end
     end
-  
-    # The Rails SVN revision, if it's checked out into vendor/rails.
+
+    # The Rails Git revision, if it's checked out into vendor/rails.
     property 'Edge Rails revision' do
       edge_rails_revision
     end
-  
+
     # The application's location on the filesystem.
     property 'Application root' do
       File.expand_path(RAILS_ROOT)
@@ -110,7 +116,7 @@ module Rails
     property 'Environment' do
       RAILS_ENV
     end
-    
+
     # The name of the database adapter for the current environment.
     property 'Database adapter' do
       ActiveRecord::Base.configurations[RAILS_ENV]['adapter']
